@@ -2,9 +2,15 @@
 
 export enum MediaType {
   JavaScript = "JavaScript",
+  Mjs = "Mjs",
+  Cjs = "Cjs",
   Jsx = "Jsx",
   TypeScript = "TypeScript",
+  Mts = "Mts",
+  Cts = "Cts",
   Dts = "Dts",
+  Dmts = "Dmts",
+  Dcts = "Dcts",
   Tsx = "Tsx",
   Json = "Json",
   Wasm = "Wasm",
