@@ -316,7 +316,6 @@ mod tests {
   use source::tests::MockResolver;
   use source::CacheInfo;
   use source::MemoryLoader;
-  use source::ResolveResult;
 
   type Sources<'a> = Vec<(
     &'a str,
@@ -379,11 +378,7 @@ mod tests {
         ModuleSpecifier::parse("file:///a/test02.ts").unwrap();
       let dependency = maybe_dependency.unwrap();
       assert!(!dependency.is_dynamic);
-      if let Resolved::Ok {
-        resolve_result: ResolveResult { specifier, .. },
-        ..
-      } = &dependency.maybe_code
-      {
+      if let Resolved::Ok { specifier, .. } = &dependency.maybe_code {
         assert_eq!(specifier, &dependency_specifier);
       } else {
         panic!("unexpected resolved slot");
@@ -1525,10 +1520,8 @@ export function a(a) {
       Some((
         "file:///a.js".to_string(),
         Resolved::Ok {
-          resolve_result: ResolveResult {
-            specifier: ModuleSpecifier::parse("file:///a.d.ts").unwrap(),
-            kind: ModuleKind::Esm,
-          },
+          specifier: ModuleSpecifier::parse("file:///a.d.ts").unwrap(),
+          kind: ModuleKind::Esm,
           range: Range {
             specifier: ModuleSpecifier::parse("file:///package.json").unwrap(),
             start: Position::zeroed(),
@@ -2635,11 +2628,7 @@ export function a(a) {
       .get("https://example.com/preact/jsx-runtime")
       .unwrap();
     assert!(!dep.maybe_code.is_none());
-    if let Resolved::Ok {
-      resolve_result: ResolveResult { specifier, .. },
-      ..
-    } = &dep.maybe_code
-    {
+    if let Resolved::Ok { specifier, .. } = &dep.maybe_code {
       assert_eq!(
         specifier,
         &ModuleSpecifier::parse("https://example.com/preact/jsx-runtime")
@@ -2746,4 +2735,41 @@ export function a(a) {
       })
     );
   }
+
+  #[test]
+  fn test_parse_ts_jsdoc_imports_ignored() {
+    let specifier = ModuleSpecifier::parse("file:///a/test.ts").unwrap();
+    let result = parse_module(
+      &specifier,
+      None,
+      Arc::new(
+        r#"
+/**
+ * Some js doc
+ *
+ * @param {import("./types.d.ts").A} a
+ * @return {import("./other.ts").B}
+ */
+export function a(a: A): B {
+  return;
+}
+"#
+        .to_string(),
+      ),
+      Some(&ModuleKind::Esm),
+      None,
+      None,
+    );
+    assert!(result.is_ok());
+    let actual = result.unwrap();
+    assert_eq!(
+      json!(actual),
+      json!({
+        "kind": "esm",
+        "mediaType": "TypeScript",
+        "size": 143,
+        "specifier": "file:///a/test.ts"
+      })
+    );
+  }
 }
